@@ -397,10 +397,14 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         :param pipeline_run_id: pipeline run id
         :return: SpanContext instance of the root span
         """
-        url = f'{self._pipeline_api_base}/pipelines/runs/{pipeline_run_id}/spans/root-span'
-        response = self._get(url)
-        response['span']['client'] = self
-        return Span(**response['span'])
+        url = f'{self._pipeline_api_base}/pipelines/runs/{pipeline_run_id}/spans'
+        params = {"onlyRootSpan": "true"}
+        response = self._get(url, params=params)
+        print(f'response: {response}')
+        return_value = dict()
+        return_value['span'] = response['spans'][0]
+        return_value['span']['client'] = self
+        return Span(**return_value['span'])
 
     # convert span object to dict type
     def convert_span_event_to_dict(self, span_event: CreateSpanEvent):
@@ -1196,115 +1200,185 @@ class TorchHttpClient(TorchClientInterFace, ABC):
         return int(round(time.time() * 1000))
 
     def _post(self, url, payload=None, as_json=True):
-        now_ms = self.now_ms()
-        if _HEADERS['accessKey'] is None:
-            _HEADERS['accessKey'] = self.access_key
-        if _HEADERS['secretKey'] is None:
-            _HEADERS['secretKey'] = self.secret_key
-        response = requests.post(
-            url=url, headers=_HEADERS, json=payload, timeout=self._timeout, verify=False
-        )
-        self.logger.info(
-            f" POST {url} "
-            f"payload={json.dumps(payload)} "
-            f"duration_ms={self.now_ms() - now_ms}"
-        )
-
+        retry = True
+        max_retry = 3
+        retry_count = 0
+        while retry:
+            now_ms = self.now_ms()
+            if _HEADERS['accessKey'] is None:
+                _HEADERS['accessKey'] = self.access_key
+            if _HEADERS['secretKey'] is None:
+                _HEADERS['secretKey'] = self.secret_key
+            response = requests.post(
+                url=url, headers=_HEADERS, json=payload, timeout=self._timeout, verify=False
+            )
+            self.logger.info(
+                f" POST {url} "
+                f"payload={json.dumps(payload)} "
+                f"duration_ms={self.now_ms() - now_ms}"
+            )
+            retry_count = retry_count + 1
+            if 500 <= response.status_code <= 511 and retry_count < max_retry:
+                retry = True
+                self.logger.info(
+                    f" Retrying POST call because of error code: {response.status_code}"
+                )
+            else:
+                retry = False
         return self._response(response, as_json)
 
     def _put(self, url, payload=None, as_json=True):
-        now_ms = self.now_ms()
-        if _HEADERS['accessKey'] is None:
-            _HEADERS['accessKey'] = self.access_key
-        if _HEADERS['secretKey'] is None:
-            _HEADERS['secretKey'] = self.secret_key
-        response = requests.put(
-            url=url, headers=_HEADERS, json=payload, timeout=self._timeout, verify=False
-        )
-        self.logger.info(
-            f" PUT {url} "
-            f"payload={json.dumps(payload)} "
-            f"duration_ms={self.now_ms() - now_ms}"
-        )
+        retry = True
+        max_retry = 3
+        retry_count = 0
+        while retry:
+            now_ms = self.now_ms()
+            if _HEADERS['accessKey'] is None:
+                _HEADERS['accessKey'] = self.access_key
+            if _HEADERS['secretKey'] is None:
+                _HEADERS['secretKey'] = self.secret_key
+            response = requests.put(
+                url=url, headers=_HEADERS, json=payload, timeout=self._timeout, verify=False
+            )
+            self.logger.info(
+                f" PUT {url} "
+                f"payload={json.dumps(payload)} "
+                f"duration_ms={self.now_ms() - now_ms}"
+            )
+            retry_count = retry_count + 1
+            if 500 <= response.status_code <= 511 and retry_count < max_retry:
+                retry = True
+                self.logger.info(
+                    f" Retrying PUT call because of error code: {response.status_code}"
+                )
+            else:
+                retry = False
 
         return self._response(response, as_json)
 
     def _get(self, url, params=None, as_json=True):
-        now_ms = self.now_ms()
-        if _HEADERS['accessKey'] is None:
-            _HEADERS['accessKey'] = self.access_key
-        if _HEADERS['secretKey'] is None:
-            _HEADERS['secretKey'] = self.secret_key
-        response = requests.get(
-            url=url, params=params, headers=_HEADERS, timeout=self._timeout, verify=False
-        )
-        self.logger.info(
-            f" GET {url} "
-            f"duration_ms={self.now_ms() - now_ms}"
-        )
+        retry = True
+        max_retry = 3
+        retry_count = 0
+        while retry:
+            now_ms = self.now_ms()
+            if _HEADERS['accessKey'] is None:
+                _HEADERS['accessKey'] = self.access_key
+            if _HEADERS['secretKey'] is None:
+                _HEADERS['secretKey'] = self.secret_key
+            response = requests.get(
+                url=url, params=params, headers=_HEADERS, timeout=self._timeout, verify=False
+            )
+            self.logger.info(
+                f" GET {url} "
+                f"duration_ms={self.now_ms() - now_ms}"
+            )
+            retry_count = retry_count + 1
+            if 500 <= response.status_code <= 511 and retry_count < max_retry:
+                retry = True
+                self.logger.info(
+                    f" Retrying GET call because of error code: {response.status_code}"
+                )
+            else:
+                retry = False
 
         return self._response(response, as_json)
 
     def _post_form_data(self, url, payload=None, as_json=True):
-        now_ms = self.now_ms()
-        if _HEADERS['accessKey'] is None:
-            _HEADERS['accessKey'] = self.access_key
-        if _HEADERS['secretKey'] is None:
-            _HEADERS['secretKey'] = self.secret_key
+        retry = True
+        max_retry = 3
+        retry_count = 0
+        while retry:
+            now_ms = self.now_ms()
+            if _HEADERS['accessKey'] is None:
+                _HEADERS['accessKey'] = self.access_key
+            if _HEADERS['secretKey'] is None:
+                _HEADERS['secretKey'] = self.secret_key
 
-        params = {
-            'request': json.dumps(payload)
-        }
-        data = MultipartEncoder(fields=params)
-        _HEADERS['Content-Type'] = data.content_type
-        response = requests.post(
-            url=url, headers=_HEADERS, data=data, timeout=self._timeout, verify=False
-        )
-        self.logger.info(
-            f" POST {url} "
-            f"payload={json.dumps(payload)} "
-            f"duration_ms={self.now_ms() - now_ms}"
-        )
+            params = {
+                'request': json.dumps(payload)
+            }
+            data = MultipartEncoder(fields=params)
+            _HEADERS['Content-Type'] = data.content_type
+            response = requests.post(
+                url=url, headers=_HEADERS, data=data, timeout=self._timeout, verify=False
+            )
+            self.logger.info(
+                f" POST {url} "
+                f"payload={json.dumps(payload)} "
+                f"duration_ms={self.now_ms() - now_ms}"
+            )
+            retry_count = retry_count + 1
+            if 500 <= response.status_code <= 511 and retry_count < max_retry:
+                retry = True
+                self.logger.info(
+                    f" Retrying POST call because of error code: {response.status_code}"
+                )
+            else:
+                retry = False
 
         return self._response(response, as_json)
 
     def _put_form_data(self, url, payload=None, as_json=True):
-        now_ms = self.now_ms()
-        if _HEADERS['accessKey'] is None:
-            _HEADERS['accessKey'] = self.access_key
-        if _HEADERS['secretKey'] is None:
-            _HEADERS['secretKey'] = self.secret_key
+        retry = True
+        max_retry = 3
+        retry_count = 0
+        while retry:
+            now_ms = self.now_ms()
+            if _HEADERS['accessKey'] is None:
+                _HEADERS['accessKey'] = self.access_key
+            if _HEADERS['secretKey'] is None:
+                _HEADERS['secretKey'] = self.secret_key
 
-        params = {
-            'request': json.dumps(payload)
-        }
-        data = MultipartEncoder(fields=params)
-        _HEADERS['Content-Type'] = data.content_type
-        response = requests.put(
-            url=url, headers=_HEADERS, data=data, timeout=self._timeout, verify=False
-        )
-        self.logger.info(
-            f" POSPUTT {url} "
-            f"payload={json.dumps(payload)} "
-            f"duration_ms={self.now_ms() - now_ms}"
-        )
+            params = {
+                'request': json.dumps(payload)
+            }
+            data = MultipartEncoder(fields=params)
+            _HEADERS['Content-Type'] = data.content_type
+            response = requests.put(
+                url=url, headers=_HEADERS, data=data, timeout=self._timeout, verify=False
+            )
+            self.logger.info(
+                f" POSPUTT {url} "
+                f"payload={json.dumps(payload)} "
+                f"duration_ms={self.now_ms() - now_ms}"
+            )
+            retry_count = retry_count + 1
+            if 500 <= response.status_code <= 511 and retry_count < max_retry:
+                retry = True
+                self.logger.info(
+                    f" Retrying POSPUTT call because of error code: {response.status_code}"
+                )
+            else:
+                retry = False
 
         return self._response(response, as_json)
 
     def _delete(self, url, params=None, as_json=True):
-        now_ms = self.now_ms()
-        if _HEADERS['accessKey'] is None:
-            _HEADERS['accessKey'] = self.access_key
-        if _HEADERS['secretKey'] is None:
-            _HEADERS['secretKey'] = self.secret_key
-        response = requests.delete(
-            url=url, params=params, headers=_HEADERS, timeout=self._timeout, verify=False
-        )
-        self.logger.info(
-            f" DELETE {url} "
-            f"duration_ms={self.now_ms() - now_ms}"
-        )
-
+        retry = True
+        max_retry = 3
+        retry_count = 0
+        while retry:
+            now_ms = self.now_ms()
+            if _HEADERS['accessKey'] is None:
+                _HEADERS['accessKey'] = self.access_key
+            if _HEADERS['secretKey'] is None:
+                _HEADERS['secretKey'] = self.secret_key
+            response = requests.delete(
+                url=url, params=params, headers=_HEADERS, timeout=self._timeout, verify=False
+            )
+            self.logger.info(
+                f" DELETE {url} "
+                f"duration_ms={self.now_ms() - now_ms}"
+            )
+            retry_count = retry_count + 1
+            if 500 <= response.status_code <= 511 and retry_count < max_retry:
+                retry = True
+                self.logger.info(
+                    f" Retrying DELETE call because of error code: {response.status_code}"
+                )
+            else:
+                retry = False
         return response
 
     def _response(self, response, as_json):
